from typing import Optional

import tcod

from actions import Action, QuitAction, MoveAction
from engine import Engine

class EventHandler(tcod.event.EventDispatch[Action]):
	def __init__(self, engine: Engine):
		super().__init__()
		self.engine = engine

	def ev_quit(self, event: tcod.event.Quit) -> Optional[Action]:
		return QuitAction()

	def ev_keydown(self, event: tcod.event.KeyDown) -> Optional[Action]:
		key = event.sym #SDL stuff, neat.

		#parse input
		match key:
			case tcod.event.KeySym.ESCAPE:
				return QuitAction()

			case tcod.event.KeySym.UP:
				return MoveAction(self.engine.player, xdir = 0, ydir = -1)
			case tcod.event.KeySym.DOWN:
				return MoveAction(self.engine.player, xdir = 0, ydir = 1)
			case tcod.event.KeySym.LEFT:
				return MoveAction(self.engine.player, xdir = -1, ydir = 0)
			case tcod.event.KeySym.RIGHT:
				return MoveAction(self.engine.player, xdir = 1, ydir = 0)

			case _:
				return None